@@ -322,110 +322,7 @@ class BertPooler(tf.keras.layers.Layer):
         return outputs
 
 
-class BertPretrainedModel(tf.keras.Model):
-
-    def __init__(self, return_states=False, return_attention_weights=False, **kwargs):
-        unused_keys = []
-        for k, v in kwargs.items():
-            if str(k).startswith('skip_'):
-                unused_keys.append(k)
-        for k in unused_keys:
-            kwargs.pop(k, None)
-        super().__init__(**kwargs)
-        self.return_states = return_states
-        self.return_attention_weights = return_attention_weights
-
-    @tf.function(input_signature=[
-        {
-            'input_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='input_ids'),
-            'segment_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='segment_ids'),
-            'attention_mask': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='attention_mask')
-        }
-    ])
-    def serving(self, inputs):
-        return self.forward(inputs)
-
-    def forward(self, inputs):
-        """Forward pass for serving.
-
-        Arguments:
-            inputs: A dict, contains `input_ids`, `segment_ids` and `attention_mask`
-
-        Returns:
-            outputs: A dict, contains outputs from serving
-        """
-        raise NotImplementedError()
-
-    def dummy_inputs(self):
-        input_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
-        segment_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
-        attn_mask = tf.constant([1] * 128, dtype=tf.int32, shape=(1, 128))
-        return input_ids, segment_ids, attn_mask
-
-    @classmethod
-    def _build_extra_config(cls, model_config, **kwargs):
-        extra_config = {}
-        for k, v in kwargs.items():
-            if k in ['verbose']:
-                continue
-            if k not in model_config:
-                if str(k).startswith('override_'):
-                    k = k.lstrip('override_')
-                extra_config[k] = v
-        if extra_config:
-            logging.info('Load extra config: \n%s', json.dumps(extra_config, indent=4))
-        return extra_config
-
-    @classmethod
-    def _merge_config(cls, model_config, extra_config):
-        mixed_config = {}
-        mixed_config.update(**model_config)
-        mixed_config.update(**extra_config)
-        logging.info('Using mixed config: \n%s', json.dumps(mixed_config, indent=4))
-        return mixed_config
-
-    @classmethod
-    def from_pretrained(cls, pretrained_model_dir, adapter=None, **kwargs):
-        config_file, ckpt, vocab_file = parse_pretrained_model_files(pretrained_model_dir)
-        if not adapter:
-            adapter = BertAdapter(**kwargs)
-        model_config = adapter.adapte_config(config_file, **kwargs)
-        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
-        extra_config = cls._build_extra_config(model_config, **kwargs)
-        mixed_config = cls._merge_config(model_config, extra_config)
-        model = cls(**mixed_config)
-        input_ids, segment_ids, attn_mask = model.dummy_inputs()
-        model(inputs=[input_ids, segment_ids, attn_mask], training=False)
-        adapter.adapte_weights(
-            model.bert_model,
-            model_config,
-            ckpt,
-            model_prefix=model.name_prefix,
-            **kwargs)
-        return model
-
-    @classmethod
-    def from_config_file(cls, config_file, **kwargs):
-        with open(config_file, mode='rt', encoding='utf-8') as fin:
-            model_config = json.load(fin)
-        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
-        extra_config = cls._build_extra_config(model_config, **kwargs)
-        mixed_config = cls._merge_config(model_config, extra_config)
-        model = cls(**mixed_config)
-        input_ids, segment_ids, attn_mask = model.dummy_inputs()
-        model(input_ids, segment_ids, attn_mask, training=False)
-        return model
-
-    @property
-    def bert_model(self):
-        return self
-
-    @property
-    def name_prefix(self):
-        return ''
-
-
-class Bert(BertPretrainedModel):
+class BertModel(tf.keras.layers.Layer):
 
     def __init__(self,
                  vocab_size=21128,
@@ -440,26 +337,8 @@ class Bert(BertPretrainedModel):
                  attention_dropout_rate=0.1,
                  initializer_range=0.02,
                  epsilon=1e-12,
-                 return_states=False,
-                 return_attention_weights=False,
                  **kwargs):
-        super().__init__(
-            return_states=return_states,
-            return_attention_weights=return_attention_weights,
-            **kwargs)
-
-        self.vocab_size = vocab_size
-        self.type_vocab_size = type_vocab_size
-        self.max_positions = max_positions
-        self.num_layers = num_layers
-        self.hidden_size = hidden_size
-        self.num_attention_heads = num_attention_heads
-        self.intermediate_size = intermediate_size
-        self.hidden_dropout_rate = hidden_dropout_rate
-        self.attention_dropout_rate = attention_dropout_rate
-        self.initializer_range = initializer_range
-        self.initialize_range = initializer_range
-
+        super().__init__(**kwargs)
         self.bert_embedding = BertEmbedding(
             vocab_size=vocab_size,
             max_positions=max_positions,
@@ -487,34 +366,146 @@ class Bert(BertPretrainedModel):
     def get_embedding_table(self):
         return self.bert_embedding.embedding_table
 
-    def forward(self, inputs):
-        input_ids, segment_ids, attention_mask = inputs['input_ids'], inputs['segment_ids'], inputs['attention_mask']
-        outputs = self(inputs=[input_ids, segment_ids, attention_mask])
-        outputs = list(outputs)
-        results = {
-            'sequence_output': outputs.pop(0),
-            'pooled_output': outputs.pop(0),
-        }
-        if self.return_states:
-            results['hidden_states'] = outputs.pop(0)
-        if self.return_attention_weights:
-            results['attention_weights'] = outputs.pop(0)
-        return results
-
-    def call(self, inputs, training=None):
-        input_ids, segment_ids, attention_mask = unpack_inputs_3(inputs)
+    def call(self, input_ids, segment_ids, attention_mask, training=None):
         embedding = self.bert_embedding(input_ids, segment_ids, training=training)
         # (batch_size, seq_len) -> (batch_size, 1, 1, seq_len)
         attention_mask = attention_mask[:, tf.newaxis, tf.newaxis, :]
         output, all_hidden_states, all_attention_scores = self.bert_encoder(
             embedding, attention_mask, training=training)
         pooled_output = self.bert_pooler(output)
-        outputs = (output, pooled_output)
-        if self.return_states:
-            outputs += (all_hidden_states, )
-        if self.return_attention_weights:
-            outputs += (all_attention_scores, )
-        return outputs
+        return output, pooled_output, all_hidden_states, all_attention_scores
+
+
+class BertPretrainedModel(tf.keras.Model):
+    """Base class for all pretrained model. Can not used to initialize an instance directly."""
+
+    @classmethod
+    def from_pretrained(
+            cls,
+            pretrained_model_dir,
+            model_params=None,
+            use_functional_api=True,
+            adapter=None,
+            check_weights=True,
+            verbose=True,
+            **kwargs):
+        config_file, ckpt, _ = parse_pretrained_model_files(pretrained_model_dir)
+        if not adapter:
+            adapter = BertAdapter(
+                skip_token_embedding=kwargs.pop('skip_token_embedding', False),
+                skip_position_embedding=kwargs.pop('skip_position_embedding', False),
+                skip_segment_embedding=kwargs.pop('skip_segment_embedding', False),
+                skip_embedding_layernorm=kwargs.pop('skip_embedding_layernorm', False),
+                skip_pooler=kwargs.pop('skip_pooler', False),
+                verbose=kwargs.pop('verbose', False),
+            )
+        model_config = adapter.adapte_config(config_file, **kwargs)
+        if model_params:
+            model_config.update(model_params)
+        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
+        model = cls(**model_config, **kwargs)
+        assert getattr(model, 'bert_model', None) is not None, 'BertPretrainedModel must have an attribute named bert_model!'
+        inputs = model.dummy_inputs()
+        model(inputs=list(inputs), training=False)
+        adapter.adapte_weights(
+            bert=model.bert_model,
+            config=model_config,
+            ckpt=ckpt,
+            prefix='' if use_functional_api else model.name,
+            check_weights=check_weights,
+            verbose=verbose,
+            **kwargs)
+        return model
+
+    @classmethod
+    def from_config_file(cls, config_file, model_params=None, adapter=None, **kwargs):
+        if not adapter:
+            adapter = BertAdapter(
+                skip_token_embedding=kwargs.pop('skip_token_embedding', False),
+                skip_position_embedding=kwargs.pop('skip_position_embedding', False),
+                skip_segment_embedding=kwargs.pop('skip_segment_embedding', False),
+                skip_embedding_layernorm=kwargs.pop('skip_embedding_layernorm', False),
+                skip_pooler=kwargs.pop('skip_pooler', False)
+            )
+        model_config = adapter.adapte_config(config_file, **kwargs)
+        if model_params:
+            model_config.update(model_params)
+        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
+        model = cls(**model_config, **kwargs)
+        assert getattr(model, 'bert_model', None) is not None, 'BertPretrainedModel must have an attribute named bert_model!'
+        inputs = model.dummy_inputs()
+        model(inputs=list(inputs), training=False)
+        return model
+
+    def dummy_inputs(self):
+        input_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
+        segment_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
+        attn_mask = tf.constant([1] * 128, dtype=tf.int32, shape=(1, 128))
+        return input_ids, segment_ids, attn_mask
+
+
+class Bert(BertPretrainedModel):
+
+    def __init__(self,
+                 vocab_size=21128,
+                 max_positions=512,
+                 hidden_size=768,
+                 type_vocab_size=2,
+                 num_layers=6,
+                 num_attention_heads=8,
+                 intermediate_size=3072,
+                 activation='gelu',
+                 hidden_dropout_rate=0.2,
+                 attention_dropout_rate=0.1,
+                 initializer_range=0.02,
+                 epsilon=1e-12,
+                 return_states=False,
+                 return_attention_weights=False,
+                 **kwargs):
+        input_ids = tf.keras.layers.Input(shape=(None, ), dtype=tf.int32, name='input_ids')
+        segment_ids = tf.keras.layers.Input(shape=(None, ), dtype=tf.int32, name='segment_ids')
+        attention_mask = tf.keras.layers.Input(shape=(None, ), dtype=tf.int32, name='attention_mask')
+        bert_model = BertModel(
+            vocab_size=vocab_size,
+            max_positions=max_positions,
+            hidden_size=hidden_size,
+            type_vocab_size=type_vocab_size,
+            num_layers=num_layers,
+            num_attention_heads=num_attention_heads,
+            intermediate_size=intermediate_size,
+            activation=activation,
+            hidden_dropout_rate=hidden_dropout_rate,
+            attention_dropout_rate=attention_dropout_rate,
+            initializer_range=initializer_range,
+            epsilon=epsilon,
+            name='bert')
+        sequence_output, pooled_output, hidden_states, attention_weights = bert_model(
+            input_ids, segment_ids, attention_mask)
+        outputs = [
+            tf.keras.layers.Lambda(lambda x: x, name='sequence_output')(sequence_output),
+            tf.keras.layers.Lambda(lambda x: x, name='pooled_output')(pooled_output),
+        ]
+        if return_states:
+            outputs += [tf.keras.layers.Lambda(lambda x: x, name='hidden_states')(hidden_states)]
+        if return_attention_weights:
+            outputs += [tf.keras.layers.Lambda(lambda x: x, name='attention_weights')(attention_weights)]
+
+        super().__init__(inputs=[input_ids, segment_ids, attention_mask], outputs=outputs, **kwargs)
+
+        self.bert_model = bert_model
+        self.vocab_size = vocab_size
+        self.type_vocab_size = type_vocab_size
+        self.max_positions = max_positions
+        self.num_layers = num_layers
+        self.hidden_size = hidden_size
+        self.num_attention_heads = num_attention_heads
+        self.intermediate_size = intermediate_size
+        self.hidden_dropout_rate = hidden_dropout_rate
+        self.attention_dropout_rate = attention_dropout_rate
+        self.initializer_range = initializer_range
+        self.initialize_range = initializer_range
+        self.return_states = return_states
+        self.return_attention_weights = return_attention_weights
 
     def get_config(self):
         config = {
@@ -529,6 +520,6 @@ class Bert(BertPretrainedModel):
             'attention_dropout_rate': self.attention_dropout_rate,
             'initializer_range': self.initializer_range,
             'return_states': self.return_states,
-            'return_attention_weights': self.return_attention_weights
+            'return_attention_weights': self.return_attention_weights,
         }
         return config
